import 'dart:math' as math;

import 'package:fis_common/env/env.dart';
import 'package:fis_ui/values/index.dart';
import 'package:flutter/material.dart';

/// FTooltip控制器
class FTooltipController extends ChangeNotifier {
  bool _isShowing = false;

  /// 是否在显示
  bool get isShowing => _isShowing;

  /// 显示
  void show() {
    _isShowing = true;
    notifyListeners();
  }

  /// 隐藏
  void hide() {
    _isShowing = false;
    notifyListeners();
  }

  /// 显示/隐藏 切换
  void toggle() {
    _isShowing = !_isShowing;
    notifyListeners();
  }
}

/// 提示工具
class FTooltip extends StatefulWidget {
  const FTooltip({
    Key? key,
    required this.child,
    this.showArrow = true,
    this.arrowColor = const Color.fromARGB(220, 0, 0, 0),
    // this.barrierColor = Colors.black12,
    this.barrierColor = Colors.transparent,
    this.arrowSize = 10.0,
    this.horizontalMargin = 10.0,
    this.verticalMargin = 10.0,
    this.triggerType = FEventTriggerType.manual,
    this.controller,
    this.position,
    this.textAlignment = FlAlignmentOnAxis.center,
    this.textLines,
    this.tooltipConentBuilder,
  })  : assert(textLines != null || tooltipConentBuilder != null),
        super(key: key);

  /// 外显组件
  final Widget child;

  /// 是否显示箭头，默认显示
  final bool showArrow;

  /// 箭头颜色
  final Color arrowColor;

  /// 屏障颜色
  final Color barrierColor;

  /// 水平外边距
  final double horizontalMargin;

  /// 垂直外边距
  final double verticalMargin;

  /// 箭头尺寸
  final double arrowSize;

  /// 触发方式
  ///
  /// 值为[FEventTriggerType.manual]时，需要通过创建[controller]控制
  final FEventTriggerType triggerType;

  /// 控制器
  final FTooltipController? controller;

  /// 显示位置
  final FVerticalPosition? position;

  /// 文本集合
  final Iterable<String>? textLines;

  /// 提示文本对齐方式
  final FlAlignmentOnAxis textAlignment;

  /// 自定义提示内容建造器
  final Widget Function()? tooltipConentBuilder;

  @override
  State<StatefulWidget> createState() => _FTooltipState();
}

class _FTooltipState extends State<FTooltip> {
  RenderBox? _childBox;
  RenderBox? _parentBox;
  OverlayEntry? _overlayEntry;
  FTooltipController? _controller;

  @override
  void initState() {
    super.initState();
    _controller = widget.controller;
    _controller ??= FTooltipController();
    _controller?.addListener(_updateView);
    WidgetsBinding.instance?.addPostFrameCallback((call) {
      if (mounted) {
        _childBox = context.findRenderObject() as RenderBox?;
        _parentBox =
            Overlay.of(context)?.context.findRenderObject() as RenderBox?;
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    var child = Material(
      child: InkWell(
        hoverColor: Colors.transparent,
        focusColor: Colors.transparent,
        splashColor: Colors.transparent,
        highlightColor: Colors.transparent,
        child: widget.child,
        onDoubleTap: widget.triggerType == FEventTriggerType.doubleClick
            ? _showView
            : null,
        onTap: widget.triggerType == FEventTriggerType.click ? _showView : null,
        onLongPress: widget.triggerType == FEventTriggerType.longPress
            ? _showView
            : null,
      ),
      color: Colors.transparent,
    );
    if (FPlatform.isIOS) {
      return child;
    } else {
      return WillPopScope(
        onWillPop: () {
          _hideView();
          return Future.value(true);
        },
        child: child,
      );
    }
  }

  void _updateView() {
    bool isShowing = _controller?.isShowing ?? false;
    if (isShowing) {
      _showView();
    } else {
      _hideView();
    }
  }

  void _showView() {
    _buildOverlayEntry();
    if (_overlayEntry != null) {
      Overlay.of(context)!.insert(_overlayEntry!);
    }
  }

  void _hideView() {
    if (_overlayEntry != null) {
      _overlayEntry?.remove();
      _overlayEntry = null;
    }
  }

  void _buildOverlayEntry() {
    Widget arrow = ClipPath(
      child: Container(
        width: widget.arrowSize,
        height: widget.arrowSize,
        color: widget.arrowColor,
      ),
      clipper: _ArrowClipper(),
    );

    _overlayEntry = OverlayEntry(
      builder: (context) {
        return Stack(
          children: <Widget>[
            GestureDetector(
              onTap: () {
                _controller?.hide();
              },
              child: Container(
                color: widget.barrierColor,
              ),
            ),
            Center(
              child: Container(
                constraints: BoxConstraints(
                  maxWidth:
                      _parentBox!.size.width - 2 * widget.horizontalMargin,
                  minWidth: 0,
                ),
                child: CustomMultiChildLayout(
                  delegate: _LayoutDelegate(
                    anchorSize: _childBox!.size,
                    anchorOffset: _childBox!.localToGlobal(
                      Offset(-widget.horizontalMargin, 0),
                    ),
                    verticalMargin: widget.verticalMargin,
                    position: widget.position,
                  ),
                  children: <Widget>[
                    if (widget.showArrow)
                      LayoutId(
                        id: _LayoutId.arrow,
                        child: arrow,
                      ),
                    if (widget.showArrow)
                      LayoutId(
                        id: _LayoutId.downArrow,
                        child: Transform.rotate(
                          angle: math.pi,
                          child: arrow,
                        ),
                      ),
                    LayoutId(
                      id: _LayoutId.content,
                      child: Column(
                        mainAxisSize: MainAxisSize.min,
                        children: <Widget>[
                          Material(
                            child: _buildTooltipConent(),
                            color: Colors.transparent,
                          ),
                        ],
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ],
        );
      },
    );
  }

  Widget _buildTooltipConent() {
    if (widget.tooltipConentBuilder != null) {
      return widget.tooltipConentBuilder!();
    }
    const textStyle = TextStyle(fontSize: 14, color: Colors.white);
    return Container(
      padding: const EdgeInsets.all(10.0),
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(8.0),
        color: widget.arrowColor,
      ),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: (widget.textLines ?? [])
            .map((e) => Text(e, style: textStyle))
            .toList(),
      ),
    );
  }
}

enum _LayoutId {
  arrow,
  downArrow,
  content,
}

enum _LayoutPosition {
  bottomLeft,
  bottomCenter,
  bottomRight,
  topLeft,
  topCenter,
  topRight,
}

class _LayoutDelegate extends MultiChildLayoutDelegate {
  _LayoutDelegate({
    required this.anchorSize,
    required this.anchorOffset,
    required this.verticalMargin,
    this.position,
  });

  final Size anchorSize;
  final Offset anchorOffset;
  final double verticalMargin;
  final FVerticalPosition? position;

  @override
  void performLayout(Size size) {
    Size contentSize = Size.zero;
    Size arrowSize = Size.zero;
    Offset contentOffset = Offset.zero;
    Offset arrowOffset = Offset.zero;

    double anchorCenterX = anchorOffset.dx + anchorSize.width / 2;
    double anchorTopY = anchorOffset.dy;
    double anchorBottomY = anchorTopY + anchorSize.height;
    _LayoutPosition menuPosition = _LayoutPosition.bottomCenter;

    if (hasChild(_LayoutId.content)) {
      contentSize = layoutChild(
        _LayoutId.content,
        BoxConstraints.loose(size),
      );
    }
    if (hasChild(_LayoutId.arrow)) {
      arrowSize = layoutChild(
        _LayoutId.arrow,
        BoxConstraints.loose(size),
      );
    }
    if (hasChild(_LayoutId.downArrow)) {
      layoutChild(
        _LayoutId.downArrow,
        BoxConstraints.loose(size),
      );
    }

    bool isTop = false;
    if (position == null) {
      // auto calculate position
      isTop = anchorBottomY > size.height / 2;
    } else {
      isTop = position == FVerticalPosition.top;
    }
    if (anchorCenterX - contentSize.width / 2 < 0) {
      menuPosition =
          isTop ? _LayoutPosition.topLeft : _LayoutPosition.bottomLeft;
    } else if (anchorCenterX + contentSize.width / 2 > size.width) {
      menuPosition =
          isTop ? _LayoutPosition.topRight : _LayoutPosition.bottomRight;
    } else {
      menuPosition =
          isTop ? _LayoutPosition.topCenter : _LayoutPosition.bottomCenter;
    }

    switch (menuPosition) {
      case _LayoutPosition.bottomCenter:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2,
          anchorBottomY + verticalMargin,
        );
        contentOffset = Offset(
          anchorCenterX - contentSize.width / 2,
          anchorBottomY + verticalMargin + arrowSize.height,
        );
        break;
      case _LayoutPosition.bottomLeft:
        arrowOffset = Offset(anchorCenterX - arrowSize.width / 2,
            anchorBottomY + verticalMargin);
        contentOffset = Offset(
          0,
          anchorBottomY + verticalMargin + arrowSize.height,
        );
        break;
      case _LayoutPosition.bottomRight:
        arrowOffset = Offset(anchorCenterX - arrowSize.width / 2,
            anchorBottomY + verticalMargin);
        contentOffset = Offset(
          size.width - contentSize.width,
          anchorBottomY + verticalMargin + arrowSize.height,
        );
        break;
      case _LayoutPosition.topCenter:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2,
          anchorTopY - verticalMargin - arrowSize.height,
        );
        contentOffset = Offset(
          anchorCenterX - contentSize.width / 2,
          anchorTopY - verticalMargin - arrowSize.height - contentSize.height,
        );
        break;
      case _LayoutPosition.topLeft:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2,
          anchorTopY - verticalMargin - arrowSize.height,
        );
        contentOffset = Offset(
          0,
          anchorTopY - verticalMargin - arrowSize.height - contentSize.height,
        );
        break;
      case _LayoutPosition.topRight:
        arrowOffset = Offset(
          anchorCenterX - arrowSize.width / 2,
          anchorTopY - verticalMargin - arrowSize.height,
        );
        contentOffset = Offset(
          size.width - contentSize.width,
          anchorTopY - verticalMargin - arrowSize.height - contentSize.height,
        );
        break;
    }
    if (hasChild(_LayoutId.content)) {
      positionChild(_LayoutId.content, contentOffset);
    }
    bool isBottom = false;
    if (_LayoutPosition.values.indexOf(menuPosition) < 3) {
      // bottom
      isBottom = true;
    }
    if (hasChild(_LayoutId.arrow)) {
      positionChild(
        _LayoutId.arrow,
        isBottom
            ? Offset(arrowOffset.dx, arrowOffset.dy + 0.1)
            : const Offset(-100, 0),
      );
    }
    if (hasChild(_LayoutId.downArrow)) {
      positionChild(
        _LayoutId.downArrow,
        !isBottom
            ? Offset(arrowOffset.dx, arrowOffset.dy - 0.1)
            : const Offset(-100, 0),
      );
    }
  }

  @override
  bool shouldRelayout(MultiChildLayoutDelegate oldDelegate) => false;
}

class _ArrowClipper extends CustomClipper<Path> {
  @override
  Path getClip(Size size) {
    Path path = Path();
    path.moveTo(0, size.height);
    path.lineTo(size.width / 2, size.height / 2);
    path.lineTo(size.width, size.height);
    return path;
  }

  @override
  bool shouldReclip(CustomClipper<Path> oldClipper) {
    return true;
  }
}
