import 'package:dropdown_button2/dropdown_button2.dart';
import 'package:flutter/material.dart';

class FSelectTestModel {
  FSelectTestModel({
    required this.code,
    required this.name,
  });

  static generateList(int count) {
    return List.generate(
      count,
      (index) => FSelectTestModel(code: "code_$index", name: "User_$index"),
    );
  }

  final String code;
  final String name;
}

/// 单选框
class FSelect<T, TValue> extends StatefulWidget {
  const FSelect({
    Key? key,
    required this.source,
    this.value,
    required this.optionValueExtractor,
    required this.optionLabelExtractor,
    this.onSelectChanged,
    this.hintText,
    this.clearable = false,
    this.isExpanded = false,
    this.width,
    this.height,
    this.decoration,
  }) : super(key: key);

  /// 数据源
  final List<T> source;

  /// 已选值
  final TValue? value;

  /// 选项值提取器
  ///
  /// [data] 选项数据源
  final TValue Function(T data) optionValueExtractor;

  /// 选项文本提取器
  ///
  /// [data] 选项数据源
  final String Function(T data) optionLabelExtractor;

  /// 选中变更事件
  ///
  /// [value] 当前已选值
  ///
  /// [index] 当前已选索引
  final void Function(TValue? value, int? index)? onSelectChanged;

  /// 选择提示文本
  final String? hintText;

  /// 是否可清除
  final bool clearable;

  /// 是否自适应 (开启后[width]和[height]不起作用)
  final bool isExpanded;

  /// 自定义宽度
  final double? width;

  /// 自定义高度
  final double? height;

  /// 装饰器
  final BoxDecoration? decoration;

  @override
  State<StatefulWidget> createState() => _FSelectState<T, TValue>();
}

class _FSelectState<T, TValue> extends State<FSelect<T, TValue>> {
  TValue? _value;

  @override
  void initState() {
    _value = widget.value;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant FSelect<T, TValue> oldWidget) {
    _value = widget.value;
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    final buttonWidth = widget.isExpanded ? null : widget.width ?? 200;
    final buttonHeight = widget.isExpanded ? null : widget.height ?? 44;
    final hint = Text(
      widget.hintText ?? "Select",
      style: const TextStyle(fontSize: 16, color: Colors.grey),
    );
    final icon = widget.clearable && _value != null
        ? GestureDetector(
            onTap: () {
              handleChange(null);
            },
            child: const Icon(Icons.close_rounded, size: 20),
          )
        : null;

    var buttonDecoration = BoxDecoration(
      border: Border.all(color: Color(0xffdcdfe6), width: 0.8),
      // border: Border.all(color: Colors.grey[200]!, width: 0.5),
      borderRadius: BorderRadius.circular(4),
    );
    if (widget.decoration != null) {
      final cfg = widget.decoration!;
      buttonDecoration = buttonDecoration.copyWith(
        color: cfg.color,
        image: cfg.image,
        border: cfg.border,
        borderRadius: cfg.borderRadius,
        backgroundBlendMode: cfg.backgroundBlendMode,
        boxShadow: cfg.boxShadow,
        shape: cfg.shape,
        gradient: cfg.gradient,
      );
    }

    return DropdownButtonHideUnderline(
      child: DropdownButton2<TValue>(
        hint: hint,
        icon: icon,
        dropdownMaxHeight: 280,
        buttonWidth: buttonWidth,
        buttonHeight: buttonHeight,
        itemHeight: 38,
        itemPadding: const EdgeInsets.symmetric(horizontal: 14, vertical: 2),
        isExpanded: widget.isExpanded,
        dropdownOverButton: false,
        scrollbarAlwaysShow: true,
        scrollbarThickness: 4,
        scrollbarRadius: const Radius.circular(2),
        buttonPadding: const EdgeInsets.symmetric(horizontal: 14, vertical: 8),
        dropdownDecoration: BoxDecoration(
          borderRadius: BorderRadius.circular(4),
        ),
        buttonDecoration: buttonDecoration,
        offset: const Offset(0, -4),
        selectedItemHighlightColor: Theme.of(context).secondaryHeaderColor,
        selectedItemBuilder: (_) => widget.source
            .map((e) => _OptionRow(
                  widget.optionLabelExtractor(e),
                  height: buttonHeight,
                ))
            .toList(),
        value: _value,
        items: buildItems(),
        onChanged: handleChange,
      ),
    );
  }

  void handleChange(TValue? value) {
    setState(() {
      _value = value;
    });
    if (widget.onSelectChanged != null) {
      if (value == null) {
        widget.onSelectChanged!.call(null, null);
      } else {
        final index = widget.source.indexWhere(
          (x) => widget.optionValueExtractor(x) == value,
        );
        widget.onSelectChanged!.call(value, index);
      }
    }
  }

  List<DropdownMenuItem<TValue>> buildItems() {
    return widget.source.map((T e) {
      final value = widget.optionValueExtractor(e);
      final active = value == _value;
      return DropdownMenuItem<TValue>(
        child: _OptionRow(widget.optionLabelExtractor(e), isActive: active),
        value: value,
      );
    }).toList();
  }
}

class _OptionRow extends StatelessWidget {
  const _OptionRow(
    this.label, {
    Key? key,
    this.isActive = false,
    this.height,
  }) : super(key: key);

  final String label;
  final bool isActive;
  final double? height;

  @override
  Widget build(BuildContext context) {
    final text = Text(
      label,
      style: TextStyle(
        fontSize: 16,
        color: isActive ? Theme.of(context).primaryColor : null,
      ),
    );
    if (height != null) {
      return Container(
        alignment: Alignment.centerLeft,
        height: height,
        child: text,
      );
    } else {
      return text;
    }
  }
}
