import 'package:flutter/widgets.dart';

import '../define.dart';

/// 通用容器
class FContainer extends Container implements FWidget {
  /// 通用容器
  FContainer({
    Key? key,
    AlignmentGeometry? alignment,
    EdgeInsetsGeometry? padding,
    Color? color,
    Decoration? decoration,
    Decoration? foregroundDecoration,
    double? width,
    double? height,
    BoxConstraints? constraints,
    EdgeInsetsGeometry? margin,
    Matrix4? transform,
    AlignmentGeometry? transformAlignment,
    FWidget? child,
    Clip clipBehavior = Clip.none,
  }) : super(
          key: key,
          alignment: alignment,
          padding: padding,
          color: color,
          decoration: decoration,
          foregroundDecoration: foregroundDecoration,
          width: width,
          height: height,
          constraints: constraints,
          margin: margin,
          transform: transform,
          transformAlignment: transformAlignment,
          child: child,
          clipBehavior: clipBehavior,
        );
}

/// 居中容器
class FCenter extends Align implements FWidget {
  /// 居中容器
  const FCenter({
    Key? key,
    double? widthFactor,
    double? heightFactor,
    FWidget? child,
  }) : super(
            key: key,
            widthFactor: widthFactor,
            heightFactor: heightFactor,
            child: child);
}

/// 列容器
class FColumn extends Flex implements FWidget {
  /// 列容器
  ///
  /// [textDirection] 排列方向
  ///
  /// [textBaseline] 对齐基线
  ///
  /// [mainAxisAlignment] 主轴对齐方式
  ///
  /// [mainAxisSize] 主轴占用空间
  ///
  /// [crossAxisAlignment] 交叉轴对齐方式
  ///
  /// [verticalDirection] 垂直排列方向
  ///
  /// [children] 子组件集合
  FColumn({
    Key? key,
    TextDirection textDirection = TextDirection.ltr,
    TextBaseline textBaseline = TextBaseline.alphabetic,
    MainAxisAlignment mainAxisAlignment = MainAxisAlignment.start,
    MainAxisSize mainAxisSize = MainAxisSize.max,
    CrossAxisAlignment crossAxisAlignment = CrossAxisAlignment.center,
    VerticalDirection verticalDirection = VerticalDirection.down,
    List<FWidget> children = const <FWidget>[],
  }) : super(
          key: key,
          direction: Axis.vertical,
          mainAxisAlignment: mainAxisAlignment,
          mainAxisSize: mainAxisSize,
          crossAxisAlignment: crossAxisAlignment,
          verticalDirection: verticalDirection,
          textDirection: textDirection,
          textBaseline: textBaseline,
          children: children,
        );
}

/// 行容器
class FRow extends Flex implements FWidget {
  /// 行容器
  ///
  /// [textDirection] 排列方向
  ///
  /// [textBaseline] 对齐基线
  ///
  /// [mainAxisAlignment] 主轴对齐方式
  ///
  /// [mainAxisSize] 主轴占用空间
  ///
  /// [crossAxisAlignment] 交叉轴对齐方式
  ///
  /// [verticalDirection] 垂直排列方向
  ///
  /// [children] 子组件集合
  FRow({
    Key? key,
    TextDirection textDirection = TextDirection.ltr,
    TextBaseline textBaseline = TextBaseline.alphabetic,
    MainAxisAlignment mainAxisAlignment = MainAxisAlignment.start,
    MainAxisSize mainAxisSize = MainAxisSize.max,
    CrossAxisAlignment crossAxisAlignment = CrossAxisAlignment.center,
    VerticalDirection verticalDirection = VerticalDirection.down,
    List<FWidget> children = const <FWidget>[],
  }) : super(
          key: key,
          direction: Axis.horizontal,
          mainAxisAlignment: mainAxisAlignment,
          mainAxisSize: mainAxisSize,
          crossAxisAlignment: crossAxisAlignment,
          verticalDirection: verticalDirection,
          textDirection: textDirection,
          textBaseline: textBaseline,
          children: children,
        );
}

/// 内间距容器
class FPadding extends Padding implements FWidget {
  /// 内间距容器
  ///
  /// [padding] 内间距
  ///
  /// [child] 子组件
  FPadding(
    EdgeInsetsGeometry padding, {
    Key? key,
    FWidget? child,
  }) : super(
          key: key,
          padding: padding,
          child: child,
        );
}

/// 尺寸限制容器
class FSizedBox extends SizedBox implements FWidget {
  /// 尺寸限制容器
  ///
  /// [width] 宽度
  ///
  /// [height] 高度
  ///
  /// [child] 子组件
  const FSizedBox({
    Key? key,
    double? width,
    double? height,
    Widget? child,
  }) : super(
          key: key,
          width: width,
          height: height,
          child: child,
        );

  /// 尺寸限制为父组件允许的最大范围
  ///
  /// [child] 子组件
  const FSizedBox.expand({Key? key, Widget? child})
      : super(
          key: key,
          width: double.infinity,
          height: double.infinity,
          child: child,
        );

  /// 尺寸限制为父组件允许的最小范围
  ///
  /// [child] 子组件
  const FSizedBox.shrink({Key? key, Widget? child})
      : super(
          key: key,
          width: 0,
          height: 0,
          child: child,
        );
}

/// 弹性布局
class FFlex extends Flex implements FWidget {
  /// 弹性布局
  ///
  /// [direction] 布局方向, 默认水平方向
  ///
  /// [textDirection] 排列方向
  ///
  /// [textBaseline] 对齐基线
  ///
  /// [mainAxisAlignment] 主轴对齐方式
  ///
  /// [mainAxisSize] 主轴占用空间
  ///
  /// [crossAxisAlignment] 交叉轴对齐方式
  ///
  /// [verticalDirection] 垂直排列方向
  ///
  /// [clipBehavior] 裁剪行为
  FFlex({
    Key? key,
    Axis direction = Axis.horizontal,
    TextDirection textDirection = TextDirection.ltr,
    TextBaseline textBaseline = TextBaseline.alphabetic,
    MainAxisAlignment mainAxisAlignment = MainAxisAlignment.start,
    MainAxisSize mainAxisSize = MainAxisSize.max,
    CrossAxisAlignment crossAxisAlignment = CrossAxisAlignment.center,
    VerticalDirection verticalDirection = VerticalDirection.down,
    Clip clipBehavior = Clip.none,
    List<FWidget> children = const <FWidget>[],
  }) : super(
          key: key,
          direction: direction,
          textDirection: textDirection,
          textBaseline: textBaseline,
          mainAxisAlignment: mainAxisAlignment,
          mainAxisSize: mainAxisSize,
          crossAxisAlignment: crossAxisAlignment,
          verticalDirection: verticalDirection,
          clipBehavior: clipBehavior,
          children: children,
        );
}

/// 控制[Row]、[Column]或[Flex]的子组件如何伸缩的组件
class FFlexible extends Flexible implements FWidget {
  /// 控制[Row]、[Column]或[Flex]的子控件如何伸缩的组件
  ///
  /// [child] 子组件
  ///
  /// [flex] 弹性因子
  ///
  /// [fit] 填充方式
  const FFlexible({
    required FWidget child,
    Key? key,
    int flex = 1,
    FlexFit fit = FlexFit.loose,
  }) : super(
          key: key,
          child: child,
          fit: fit,
          flex: flex,
        );
}

/// 展开[Row]、[Column]或[Flex]子组件的组件，使其子组件填满可用的空间
class FExpanded extends FFlexible implements FWidget {
  /// 展开[Row]、[Column]或[Flex]子组件的组件，使其子组件填满可用的空间
  ///
  /// [child] 子组件
  ///
  /// [flex] 弹性因子
  const FExpanded({
    required FWidget child,
    Key? key,
    int flex = 1,
  }) : super(
          key: key,
          child: child,
          flex: flex,
          fit: FlexFit.tight,
        );
}

/// 堆叠布局
class FStack extends Stack implements FWidget {
  /// 堆叠布局
  ///
  /// [children] 子组件集合
  ///
  /// [textDirection] 排列方向
  ///
  /// [alignment] 对齐方式
  ///
  /// [fit] 约束方式
  ///
  /// [clipBehavior] 裁剪行为
  FStack({
    Key? key,
    TextDirection? textDirection,
    AlignmentGeometry alignment = AlignmentDirectional.topStart,
    StackFit fit = StackFit.loose,
    Clip clipBehavior = Clip.hardEdge,
    List<FWidget> children = const <FWidget>[],
  }) : super(
          key: key,
          textDirection: textDirection,
          alignment: alignment,
          fit: fit,
          clipBehavior: clipBehavior,
          children: children,
        );
}

/// 对子组件施加额外约束的组件
class FConstrainedBox extends ConstrainedBox implements FWidget {
  /// 对子组件施加额外约束的组件
  ///
  /// [constraints] 约束
  ///
  /// [child] 子组件
  FConstrainedBox({
    required BoxConstraints constraints,
    Key? key,
    FWidget? child,
  }) : super(
          key: key,
          child: child,
          constraints: constraints,
        );
}

/// 对子元素进行合适的缩放和定位
class FFittedBox extends FittedBox implements FWidget {
  const FFittedBox({
    Key? key,
    BoxFit fit = BoxFit.contain,
    Alignment alignment = Alignment.center,
    Clip clipBehavior = Clip.none,
    Widget? child,
  }) : super(
          key: key,
          fit: fit,
          alignment: alignment,
          clipBehavior: clipBehavior,
          child: child,
        );
}

/// 用于控制[FStack]的子组件的位置。
class FPositioned extends Positioned implements FWidget {
  const FPositioned({
    Key? key,
    double? left,
    double? top,
    double? right,
    double? bottom,
    double? width,
    double? height,
    required FWidget child,
  }) : super(
          key: key,
          child: child,
          left: left,
          right: right,
          top: top,
          bottom: bottom,
          width: width,
          height: height,
        );
}
