import 'package:fis_ui/define.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';

/// 卡片组件
class FMaterialCard extends Card implements FWidget {
  FMaterialCard({
    Key? key,
    Color? color,
    Color? shadowColor,
    double? elevation,
    ShapeBorder? shape,
    bool borderOnForeground = true,
    EdgeInsetsGeometry? margin,
    Clip? clipBehavior,
    FWidget? child,
    bool semanticContainer = true,
  }) : super(
          key: key,
          color: color,
          shadowColor: shadowColor,
          elevation: elevation,
          shape: shape,
          borderOnForeground: borderOnForeground,
          margin: margin,
          clipBehavior: clipBehavior,
          child: child,
          semanticContainer: semanticContainer,
        );
}

/// 卡片组件
class FCard extends StatefulWidget implements FWidget {
  /// 标题栏，不为空时[title]无效。([title]&[header]均为空时不显示标题栏)
  final Widget? header;

  /// 内容主体
  final Widget? body;

  /// 标题，[header]为空时生效。([title]&[header]均为空时不显示标题栏)
  final String? title;

  /// 隐藏边框
  final bool? disBorder;

  /// 禁用Hover效果
  final bool? disHover;

  /// 禁用水波纹效果
  final bool? disInk;

  /// 外边缘空白
  final EdgeInsetsGeometry? margin;

  /// 卡片组件
  const FCard({
    Key? key,
    this.header,
    this.body,
    this.title,
    this.disBorder,
    this.disHover,
    this.disInk,
    this.margin,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _FCardState();
}

class _FCardState extends State<FCard> {
  static const bool semanticContainer = true;
  static const EdgeInsets headerPadding =
      EdgeInsets.symmetric(vertical: 14, horizontal: 16);
  static const EdgeInsets bodyPadding = EdgeInsets.all(16);

  late final Color _borderColor;
  // late final FCardThemeData _cardThemeData;
  bool _isHover = false;

  @override
  void initState() {
    // var themeData = FTheme.of(context);
    // _cardThemeData = themeData.cardThemeData;
    // _borderColor =
    //     _cardThemeData.borderColor ?? themeData.colorDefinition.borderBase;
    _borderColor = Colors.grey[300]!;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    var borderRadius = const BorderRadius.all(Radius.circular(4));
    Border? border =
        widget.disBorder == true ? null : Border.all(color: _borderColor);
    List<BoxShadow>? boxShadows = widget.disHover == true
        ? null
        : [
            if (_isHover)
              BoxShadow(
                color: Colors.black.withOpacity(.2),
                offset: Offset(1.6, 2.0), //阴影偏移量
                blurRadius: 6, //阴影模糊程度
                spreadRadius: 0, //阴影扩散程度
              ),
          ];

    BoxDecoration boxDecoration = BoxDecoration(
      borderRadius: borderRadius,
      boxShadow: boxShadows,
      border: border,
    );

    Widget feedbackBody;
    if (widget.disInk == true) {
      feedbackBody = MouseRegion(
        onEnter: (e) {
          _onHoverChange(true);
        },
        onExit: (e) {
          _onHoverChange(false);
        },
        cursor: SystemMouseCursors.basic,
        child: _buildBody(),
      );
    } else {
      feedbackBody = Ink(
        child: InkWell(
          onTap: () {},
          onHover: _onHoverChange,
          hoverColor: Colors.transparent,
          splashColor: Colors.grey[100],
          highlightColor: Colors.grey[100],
          mouseCursor: SystemMouseCursors.basic,
          child: _buildBody(),
        ),
      );
    }

    var result = Container(
      margin: widget.margin ?? const EdgeInsets.all(8),
      decoration: boxDecoration,
      child: Material(
        type: MaterialType.card,
        clipBehavior: Clip.none,
        borderRadius: borderRadius,
        child: feedbackBody,
      ),
    );

    return Semantics(
      container: semanticContainer,
      child: Semantics(
        explicitChildNodes: !semanticContainer,
        child: result,
      ),
    );
  }

  Widget? _buildBody() {
    Widget? header = _buildHeader();
    Widget mainBody = Container(
      alignment: Alignment.center,
      padding: bodyPadding,
      child: widget.body,
    );
    if (header != null) {
      return Column(
        children: [
          Container(
            alignment: Alignment.centerLeft,
            padding: headerPadding,
            child: header,
          ),
          Divider(color: _borderColor, height: 2),
          mainBody,
        ],
      );
    } else {
      return mainBody;
    }
  }

  Widget? _buildHeader() {
    if (widget.header != null) {
      return widget.header;
    } else if (widget.title != null && widget.title!.isNotEmpty) {
      return Text(
        widget.title!,
        style: TextStyle(
          // color: _cardThemeData.titleColor ??
          //     FTheme.of(context).colorDefinition.textPrimary,
          // fontSize: _cardThemeData.titleSize ?? 16.0,
          color: Colors.black87,
          fontSize: 16.0,
          fontWeight: FontWeight.bold,
        ),
      );
    } else {
      return null;
    }
  }

  void _onHoverChange(hover) {
    if (widget.disHover == true) return;
    setState(() {
      _isHover = hover;
    });
  }
}
