import 'dart:io';
import 'dart:typed_data';
import 'dart:ui';

import 'package:fis_ui/define.dart';
import 'package:fis_theme/theme.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

/// 图片组件
class FImage extends Image implements FWidget {
  /// 图片组件
  ///
  ///  * [new FImage],用于从[ImageProvider]获取图像.
  ///  * [FImage.asset], 用于通过Key从[AssetBundle]获取图像.
  ///  * [FImage.network], 用于从网络获取图像
  ///  * [Image.file], 用于从[File]获取图像.
  ///  * [Image.memory], 用于从[Uint8List]获取图像.
  const FImage({
    Key? key,
    required this.image,
    this.width,
    this.height,
    this.alignment = Alignment.center,
    this.fit,
    this.filterQuality = FilterQuality.low,
  }) : super(key: key, image: image);

  /// 创建从网络获取[ImageStream]的FImage组件
  ///
  /// [src] 网络图像链接
  ///
  /// [scale] 缩放比
  static FImage network(
    String src, {
    Key? key,
    double scale = 1.0,
    double? width,
    double? height,
    AlignmentGeometry alignment = Alignment.center,
    BoxFit? fit,
    FilterQuality filterQuality = FilterQuality.low,
    Map<String, String>? headers,
    int? cacheWidth,
    int? cacheHeight,
  }) {
    var image = ResizeImage.resizeIfNeeded(cacheWidth, cacheHeight,
        NetworkImage(src, scale: scale, headers: headers));
    return FImage(
      key: key,
      image: image,
      width: width,
      height: height,
      fit: fit,
      filterQuality: filterQuality,
    );
  }

  /// 创建从[File]获取[ImageStream]的FImage组件
  ///
  /// [file] 文件[File]对象
  ///
  /// [scale] 缩放比
  static FImage file(
    File file, {
    Key? key,
    double scale = 1.0,
    double? width,
    double? height,
    AlignmentGeometry alignment = Alignment.center,
    BoxFit? fit,
    FilterQuality filterQuality = FilterQuality.low,
    int? cacheWidth,
    int? cacheHeight,
  }) {
    var image = ResizeImage.resizeIfNeeded(
        cacheWidth, cacheHeight, FileImage(file, scale: scale));
    return FImage(
      key: key,
      image: image,
      width: width,
      height: height,
      fit: fit,
      filterQuality: filterQuality,
    );
  }

  /// 创建从assets资源包获取[ImageStream]的FImage组件
  ///
  /// [bundle] 资源包
  ///
  /// [package] 库包
  ///
  /// [scale] 缩放比
  static FImage asset(
    String name, {
    Key? key,
    double? scale,
    AssetBundle? bundle,
    String? package,
    double? width,
    double? height,
    AlignmentGeometry alignment = Alignment.center,
    BoxFit? fit,
    FilterQuality filterQuality = FilterQuality.low,
    int? cacheWidth,
    int? cacheHeight,
  }) {
    var image = ResizeImage.resizeIfNeeded(
        cacheWidth,
        cacheHeight,
        scale != null
            ? ExactAssetImage(name,
                bundle: bundle, scale: scale, package: package)
            : AssetImage(name, bundle: bundle, package: package));
    return FImage(
      key: key,
      image: image,
      width: width,
      height: height,
      fit: fit,
      filterQuality: filterQuality,
    );
  }

  /// 创建从[Uint8List]获取[ImageStream]的FImage组件
  ///
  /// [bytes] 已编码的图像字节，默认支持JPEG、PNG、GIF、Animated GIF、WebP、Animated WebP、BMP和WBMP
  ///
  /// [scale] 缩放比
  static FImage memory(
    Uint8List bytes, {
    Key? key,
    double scale = 1.0,
    double? width,
    double? height,
    AlignmentGeometry alignment = Alignment.center,
    BoxFit? fit,
    FilterQuality filterQuality = FilterQuality.low,
    int? cacheWidth,
    int? cacheHeight,
  }) {
    var image = ResizeImage.resizeIfNeeded(
        cacheWidth, cacheHeight, MemoryImage(bytes, scale: scale));
    return FImage(
      key: key,
      image: image,
      width: width,
      height: height,
      fit: fit,
      filterQuality: filterQuality,
    );
  }

  /// 从资源中获取图片
  ///
  /// [name] 资源Key
  factory FImage.resource(
    FResourceKey resourceKey, {
    Key? key,
    double scale = 1.0,
    double? width,
    double? height,
    AlignmentGeometry alignment = Alignment.center,
    BoxFit? fit,
    FilterQuality filterQuality = FilterQuality.low,
    int? cacheWidth,
    int? cacheHeight,
  }) {
    var image = ResizeImage.resizeIfNeeded(
        cacheWidth, cacheHeight, FResourceImage(resourceKey, scale: scale));
    return FImage(
      key: key,
      image: image,
      width: width,
      height: height,
      fit: fit,
      filterQuality: filterQuality,
    );
  }

  /// 图像内容提供者
  final ImageProvider image;

  /// 指定宽度
  final double? width;

  /// 指定高度
  final double? height;

  /// 对齐方式
  final AlignmentGeometry alignment;

  /// 适应方式
  final BoxFit? fit;

  /// 滤镜质量等级
  final FilterQuality filterQuality;
}
