import 'dart:async';

import 'package:flutter/material.dart';

/// Loading组件
class FLoading {
  static Color _backgroundColor = Colors.black26.withOpacity(.4);
  static Widget? _loadingWidget;

  static bool _loading = false;
  static BuildContext? _cacheContext;
  static Timer? _timer;

  /// 初始化默认样式
  ///
  /// [backgroundColor]背景色
  ///
  /// [loadingWidget]默认Loading样式组件
  static void init({
    Color? backgroundColor,
    Widget? loadingWidget,
  }) {
    if (backgroundColor != null) {
      _backgroundColor = backgroundColor;
    }
    _loadingWidget = loadingWidget;
  }

  /// 显示Loading
  ///
  /// [context]调用组件上下文
  ///
  /// [config]可选配置 - [FLoadingConfig]
  static void show(BuildContext context, {FLoadingConfig? config}) {
    if (!_loading) {
      _loading = true;
      showGeneralDialog(
          context: context,
          barrierDismissible: false,
          barrierLabel:
              MaterialLocalizations.of(context).modalBarrierDismissLabel,
          barrierColor: _backgroundColor,
          transitionDuration: const Duration(milliseconds: 150),
          transitionBuilder: (BuildContext context, Animation<double> animation,
              Animation<double> secondaryAnimation, Widget child) {
            return FadeTransition(
              opacity: CurvedAnimation(
                parent: animation,
                curve: Curves.easeOut,
              ),
              child: child,
            );
          },
          useRootNavigator: true,
          routeSettings: null,
          pageBuilder: (BuildContext context, Animation animation,
              Animation secondaryAnimation) {
            final Widget pageChild = Builder(builder: (context) {
              _cacheContext = context;
              Widget widget = config?.widget ??
                  (_loadingWidget ?? CircularProgressIndicator());
              return WillPopScope(
                onWillPop: () async {
                  if (config?.closable == true) {
                    hide();
                  }
                  return Future.value(false);
                },
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [widget],
                ),
              );
            });
            return pageChild;
          }).then((v) {
        _loading = false;
      });

      if (config?.duration != null) {
        _timer = Timer(Duration(milliseconds: config!.duration!), () {
          hide();
        });
      }
    }
  }

  /// 关闭Loading
  static void hide({BuildContext? context}) {
    _timer?.cancel();
    _timer = null;
    if (context != null) {
      _cacheContext = context;
    }
    if (_loading && _cacheContext != null) {
      Navigator.of(_cacheContext!).pop();
      _cacheContext = null;
    }
  }
}

class FLoadingConfig {
  final Widget? widget;
  final int? duration;
  final bool? closable;

  /// Loading配置
  ///
  /// [widget]自定义样式组件
  ///
  /// [duration]自动关闭时间
  ///
  /// [closable]是否可通过返回硬件按钮关闭
  const FLoadingConfig({
    this.widget,
    this.duration,
    this.closable,
  });
}
