import 'dart:async';
import 'dart:collection';
import 'dart:ui';

import 'package:flutter/material.dart';

enum FToastPosition {
  top,
  center,
  bottom,
}

class FToast {
  static int count = 0;
  static const double _msgFontSize = 16;
  static const double _subMsgFontSize = 13;
  static TextStyle _defaultStyle = const TextStyle(
    color: Colors.white,
    fontSize: _msgFontSize,
    height: 1.0,
  );
  static Queue<_FToastData>? _queue;
  static _FToastData? _current;

  static void show(
    final BuildContext context, {
    final String msg = "",
    final TextStyle? msgStyle,
    final String? subMsg,
    final TextStyle? subMsgStyle,
    final double subMsgSpace = 12.0,
    final double corner = 4.0,
    final Color color = Colors.black54,
    final int duration = 1500,
    final EdgeInsets padding =
        const EdgeInsets.symmetric(vertical: 14.0, horizontal: 18.0),
    final FToastPosition position = FToastPosition.center,
    final Widget? image,
    final AxisDirection imageDirection = AxisDirection.up,
    final double imageSpace = 8.0,
  }) {
    Widget buildToast() {
      List<Widget> children = [];

      /// Image on Top
      if (image != null && imageDirection == AxisDirection.up) {
        children.add(image);
        children.add(SizedBox(height: imageSpace));
      }

      /// Main Message
      var msgText = Text(msg, style: msgStyle ?? _defaultStyle);

      /// Image on Horizontal
      if (image != null && imageDirection == AxisDirection.left) {
        children.add(Row(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            image,
            SizedBox(width: imageSpace),
            msgText,
          ],
        ));
      } else if (image != null && imageDirection == AxisDirection.right) {
        children.add(Row(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            msgText,
            SizedBox(width: imageSpace),
            image,
          ],
        ));
      } else {
        children.add(msgText);
      }

      /// Sub Message
      if (subMsg != null) {
        children.add(SizedBox(height: subMsgSpace));
        var _subMsgStyle =
            subMsgStyle ?? _defaultStyle.copyWith(fontSize: _subMsgFontSize);
        children.add(Text(subMsg, style: _subMsgStyle));
      }

      /// Image on Bottom
      if (image != null && imageDirection == AxisDirection.down) {
        children.add(SizedBox(height: imageSpace));
        children.add(image);
      }

      return ConstrainedBox(
        constraints: BoxConstraints(
            minWidth: 80.0,
            minHeight: 38.0,
            maxWidth: MediaQueryData.fromWindow(window).size.width - 48.0),
        child: Container(
          padding: padding,
          decoration: BoxDecoration(
            color: color,
            borderRadius: BorderRadius.all(Radius.circular(corner)),
          ),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: children,
          ),
        ),
      );
    }

    OverlayEntry entry = OverlayEntry(
      builder: (context) {
        Widget result = IgnorePointer(
          child: Material(
            color: Colors.transparent,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [buildToast()],
            ),
          ),
        );
        if (position != FToastPosition.center) {
          return Positioned(
            width: MediaQuery.of(context).size.width,
            top: _getToastPosition(context, position),
            child: result,
          );
        } else {
          return result;
        }
      },
    );
    _FToastData toastData = _FToastData(context, duration, entry);
    _push(toastData);
  }

  static void _push(_FToastData data) {
    if (_queue == null) {
      _queue = Queue();
    }
    _queue!.add(data);

    _checkQueue();
  }

  static void _checkQueue() {
    if (_current != null || _queue!.isEmpty) return;

    _current = _queue!.removeFirst();
    try {
      _current!.show(() {
        _current = null;
        _checkQueue();
      });
    } catch (e) {
      _current = null;
    }
  }

  // 设置toast位置
  static double _getToastPosition(
      BuildContext context, FToastPosition position) {
    if (position == FToastPosition.top) {
      return MediaQuery.of(context).size.height * 1 / 6;
    } else if (position == FToastPosition.bottom) {
      return MediaQuery.of(context).size.height * 5 / 6;
    }
    return MediaQuery.of(context).size.height / 2;
  }
}

class _FToastData {
  BuildContext context;
  int duration;
  OverlayEntry entry;

  bool _disposed = false;

  _FToastData(this.context, this.duration, this.entry);

  void show(Function? callback) {
    try {
      if (context.findRenderObject()?.attached ?? false) {
        OverlayState? overlayState = Overlay.of(this.context);
        if (overlayState != null) {
          overlayState.insert(this.entry);
          Timer(Duration(milliseconds: this.duration), () {
            this._dispose();
            if (callback != null) {
              callback();
            }
          });
          return;
        }
      }
    } catch (e) {
      print(e);
    }

    this._dispose();
  }

  _dispose() {
    if (!_disposed) {
      entry.remove();
      _disposed = true;
    }
  }
}
